#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.nextDev = void 0;
var _indexJs = _interopRequireDefault(require("next/dist/compiled/arg/index.js"));
var _startServer = require("../server/lib/start-server");
var _utils = require("../server/lib/utils");
var Log = _interopRequireWildcard(require("../build/output/log"));
var _output = require("../build/output");
var _isError = _interopRequireDefault(require("../lib/is-error"));
var _getProjectDir = require("../lib/get-project-dir");
var _constants = require("../shared/lib/constants");
var _path = _interopRequireDefault(require("path"));
var _shared = require("../trace/shared");
var _net = require("net");
var _cluster = _interopRequireDefault(require("cluster"));
var _storage = require("../telemetry/storage");
var _config = _interopRequireDefault(require("../server/config"));
var _findPagesDir = require("../lib/find-pages-dir");
var _fileExists = require("../lib/file-exists");
var _watchpack = _interopRequireDefault(require("next/dist/compiled/watchpack"));
var _stripAnsi = _interopRequireDefault(require("next/dist/compiled/strip-ansi"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache() {
    if (typeof WeakMap !== "function") return null;
    var cache = new WeakMap();
    _getRequireWildcardCache = function() {
        return cache;
    };
    return cache;
}
function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache();
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {};
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
let isTurboSession = false;
let sessionStopHandled = false;
let sessionStarted = Date.now();
let dir;
let unwatchConfigFiles;
const handleSessionStop = async ()=>{
    if (sessionStopHandled) return;
    sessionStopHandled = true;
    try {
        const { eventCliSession  } = require("../telemetry/events/session-stopped");
        const config = await (0, _config).default(_constants.PHASE_DEVELOPMENT_SERVER, dir);
        let telemetry = _shared.traceGlobals.get("telemetry") || new _storage.Telemetry({
            distDir: _path.default.join(dir, config.distDir)
        });
        let appDir = !!_shared.traceGlobals.get("pagesDir");
        let pagesDir = !!_shared.traceGlobals.get("appDir");
        if (typeof _shared.traceGlobals.get("pagesDir") === "undefined" || typeof _shared.traceGlobals.get("appDir") === "undefined") {
            const pagesResult = await (0, _findPagesDir).findPagesDir(dir, !!config.experimental.appDir);
            appDir = !!pagesResult.appDir;
            pagesDir = !!pagesResult.pagesDir;
        }
        telemetry.record(eventCliSession({
            cliCommand: "dev",
            turboFlag: isTurboSession,
            durationMilliseconds: Date.now() - sessionStarted,
            pagesDir,
            appDir
        }), true);
        telemetry.flushDetached("dev", dir);
    } catch (_) {
    // errors here aren't actionable so don't add
    // noise to the output
    }
    process.exit(0);
};
if (_cluster.default.isMaster) {
    process.on("SIGINT", handleSessionStop);
    process.on("SIGTERM", handleSessionStop);
} else {
    process.on("SIGINT", ()=>process.exit(0));
    process.on("SIGTERM", ()=>process.exit(0));
}
function watchConfigFiles(dirToWatch) {
    if (unwatchConfigFiles) {
        unwatchConfigFiles();
    }
    const wp = new _watchpack.default();
    wp.watch({
        files: _constants.CONFIG_FILES.map((file)=>_path.default.join(dirToWatch, file))
    });
    wp.on("change", (filename)=>{
        console.log(`\n> Found a change in ${_path.default.basename(filename)}. Restart the server to see the changes in effect.`);
    });
    return ()=>wp.close();
}
const nextDev = async (argv)=>{
    const validArgs = {
        // Types
        "--help": Boolean,
        "--port": Number,
        "--hostname": String,
        "--turbo": Boolean,
        // To align current messages with native binary.
        // Will need to adjust subcommand later.
        "--show-all": Boolean,
        "--root": String,
        // Aliases
        "-h": "--help",
        "-p": "--port",
        "-H": "--hostname"
    };
    let args;
    try {
        args = (0, _indexJs).default(validArgs, {
            argv
        });
    } catch (error) {
        if ((0, _isError).default(error) && error.code === "ARG_UNKNOWN_OPTION") {
            return (0, _utils).printAndExit(error.message, 1);
        }
        throw error;
    }
    if (args["--help"]) {
        console.log(`
      Description
        Starts the application in development mode (hot-code reloading, error
        reporting, etc.)

      Usage
        $ next dev <dir> -p <port number>

      <dir> represents the directory of the Next.js application.
      If no directory is provided, the current directory will be used.

      Options
        --port, -p      A port number on which to start the application
        --hostname, -H  Hostname on which to start the application (default: 0.0.0.0)
        --help, -h      Displays this message
    `);
        process.exit(0);
    }
    dir = (0, _getProjectDir).getProjectDir(process.env.NEXT_PRIVATE_DEV_DIR || args._[0]);
    unwatchConfigFiles = watchConfigFiles(dir);
    // Check if pages dir exists and warn if not
    if (!await (0, _fileExists).fileExists(dir, "directory")) {
        (0, _utils).printAndExit(`> No such directory exists as the project root: ${dir}`);
    }
    async function preflight() {
        const { getPackageVersion  } = await Promise.resolve(require("../lib/get-package-version"));
        const [sassVersion, nodeSassVersion] = await Promise.all([
            getPackageVersion({
                cwd: dir,
                name: "sass"
            }),
            getPackageVersion({
                cwd: dir,
                name: "node-sass"
            }), 
        ]);
        if (sassVersion && nodeSassVersion) {
            Log.warn("Your project has both `sass` and `node-sass` installed as dependencies, but should only use one or the other. " + "Please remove the `node-sass` dependency from your project. " + " Read more: https://nextjs.org/docs/messages/duplicate-sass");
        }
    }
    const port = (0, _utils).getPort(args);
    // If neither --port nor PORT were specified, it's okay to retry new ports.
    const allowRetry = args["--port"] === undefined && process.env.PORT === undefined;
    // We do not set a default host value here to prevent breaking
    // some set-ups that rely on listening on other interfaces
    const host = args["--hostname"];
    const devServerOptions = {
        allowRetry,
        dev: true,
        dir,
        hostname: host,
        isNextDevCommand: true,
        port
    };
    // check for babelrc, swc plugins
    async function validateNextConfig(isCustomTurbopack) {
        const { getPkgManager  } = require("../lib/helpers/get-pkg-manager");
        const { getBabelConfigFile  } = require("../build/webpack-config");
        const { defaultConfig  } = require("../server/config-shared");
        const chalk = require("next/dist/compiled/chalk");
        const { interopDefault  } = require("../lib/interop-default");
        // To regenerate the TURBOPACK gradient require('gradient-string')('blue', 'red')('>>> TURBOPACK')
        const isTTY = process.stdout.isTTY;
        const turbopackGradient = `${chalk.bold(isTTY ? "\x1b[38;2;0;0;255m>\x1b[39m\x1b[38;2;23;0;232m>\x1b[39m\x1b[38;2;46;0;209m>\x1b[39m \x1b[38;2;70;0;185mT\x1b[39m\x1b[38;2;93;0;162mU\x1b[39m\x1b[38;2;116;0;139mR\x1b[39m\x1b[38;2;139;0;116mB\x1b[39m\x1b[38;2;162;0;93mO\x1b[39m\x1b[38;2;185;0;70mP\x1b[39m\x1b[38;2;209;0;46mA\x1b[39m\x1b[38;2;232;0;23mC\x1b[39m\x1b[38;2;255;0;0mK\x1b[39m" : ">>> TURBOPACK")} ${chalk.dim("(alpha)")}\n\n`;
        let thankYouMsg = `Thank you for trying Next.js v13 with Turbopack! As a reminder,\nTurbopack is currently in alpha and not yet ready for production.\nWe appreciate your ongoing support as we work to make it ready\nfor everyone.\n`;
        let unsupportedParts = "";
        let babelrc = await getBabelConfigFile(dir);
        if (babelrc) babelrc = _path.default.basename(babelrc);
        let hasNonDefaultConfig;
        let rawNextConfig = {};
        try {
            rawNextConfig = interopDefault(await (0, _config).default(_constants.PHASE_DEVELOPMENT_SERVER, dir, undefined, true));
            if (typeof rawNextConfig === "function") {
                rawNextConfig = rawNextConfig(_constants.PHASE_DEVELOPMENT_SERVER, {
                    defaultConfig
                });
            }
            const checkUnsupportedCustomConfig = (configKey = "", parentUserConfig, parentDefaultConfig)=>{
                try {
                    // these should not error
                    if (configKey === "serverComponentsExternalPackages" || configKey === "appDir" || configKey === "images" || configKey === "reactStrictMode" || configKey === "swcMinify" || configKey === "configFileName") {
                        return false;
                    }
                    let userValue = parentUserConfig == null ? void 0 : parentUserConfig[configKey];
                    let defaultValue = parentDefaultConfig == null ? void 0 : parentDefaultConfig[configKey];
                    if (typeof defaultValue !== "object") {
                        return defaultValue !== userValue;
                    }
                    return Object.keys(userValue || {}).some((key)=>{
                        return checkUnsupportedCustomConfig(key, userValue, defaultValue);
                    });
                } catch (e) {
                    console.error(`Unexpected error occurred while checking ${configKey}`, e);
                    return false;
                }
            };
            hasNonDefaultConfig = Object.keys(rawNextConfig).some((key)=>checkUnsupportedCustomConfig(key, rawNextConfig, defaultConfig));
        } catch (e) {
            console.error("Unexpected error occurred while checking config", e);
        }
        const hasWarningOrError = babelrc || hasNonDefaultConfig;
        if (!hasWarningOrError) {
            thankYouMsg = chalk.dim(thankYouMsg);
        }
        console.log(turbopackGradient + thankYouMsg);
        let feedbackMessage = `Learn more about Next.js v13 and Turbopack: ${chalk.underline("https://nextjs.link/with-turbopack")}\nPlease direct feedback to: ${chalk.underline("https://nextjs.link/turbopack-feedback")}\n`;
        if (!hasWarningOrError) {
            feedbackMessage = chalk.dim(feedbackMessage);
        }
        if (babelrc) {
            unsupportedParts += `\n- Babel detected (${chalk.cyan(babelrc)})\n  ${chalk.dim(`Babel is not yet supported. To use Turbopack at the moment,\n  you'll need to remove your usage of Babel.`)}`;
        }
        if (hasNonDefaultConfig) {
            unsupportedParts += `\n\n- Unsupported Next.js configuration option(s) (${chalk.cyan("next.config.js")})\n  ${chalk.dim(`The only configurations options supported are:\n${[
                "reactStrictMode",
                "experimental.appDir",
                "experimental.serverComponentsExternalPackages",
                "images",
                "swcMinify",
                "configFileName", 
            ].map((name)=>`    - ${chalk.cyan(name)}\n`).join("")}  To use Turbopack, remove other configuration options.`)}   `;
        }
        if (unsupportedParts) {
            const pkgManager = getPkgManager(dir);
            console.error(`${chalk.bold.red("Error:")} You are using configuration and/or tools that are not yet\nsupported by Next.js v13 with Turbopack:\n${unsupportedParts}\n
If you cannot make the changes above, but still want to try out\nNext.js v13 with Turbopack, create the Next.js v13 playground app\nby running the following commands:

  ${chalk.bold.cyan(`${pkgManager === "npm" ? "npx create-next-app" : `${pkgManager} create next-app`} --example with-turbopack with-turbopack-app`)}\n  cd with-turbopack-app\n  ${pkgManager} run dev
        `);
            console.warn(feedbackMessage);
            if (!isCustomTurbopack) {
                process.exit(1);
            } else {
                console.warn(`\n${chalk.bold.yellow("Warning:")} Unsupported config found; but continuing with custom Turbopack binary.\n`);
            }
        }
        console.log(feedbackMessage);
        return rawNextConfig;
    }
    if (args["--turbo"]) {
        var ref;
        isTurboSession = true;
        const { loadBindings , __isCustomTurbopackBinary  } = require("../build/swc");
        const { eventCliSession  } = require("../telemetry/events/version");
        const { setGlobal  } = require("../trace");
        require("../telemetry/storage");
        const findUp = require("next/dist/compiled/find-up");
        const isCustomTurbopack = await __isCustomTurbopackBinary();
        const rawNextConfig = await validateNextConfig(isCustomTurbopack);
        const distDir = _path.default.join(dir, rawNextConfig.distDir || ".next");
        const { pagesDir , appDir  } = (0, _findPagesDir).findPagesDir(dir, !!((ref = rawNextConfig.experimental) == null ? void 0 : ref.appDir));
        const telemetry = new _storage.Telemetry({
            distDir
        });
        setGlobal("appDir", appDir);
        setGlobal("pagesDir", pagesDir);
        setGlobal("telemetry", telemetry);
        if (!isCustomTurbopack) {
            telemetry.record(eventCliSession(distDir, rawNextConfig, {
                webpackVersion: 5,
                cliCommand: "dev",
                isSrcDir: _path.default.relative(dir, pagesDir || appDir || "").startsWith("src"),
                hasNowJson: !!await findUp("now.json", {
                    cwd: dir
                }),
                isCustomServer: false,
                turboFlag: true,
                pagesDir: !!pagesDir,
                appDir: !!appDir
            }));
        }
        const turboJson = findUp.sync("turbo.json", {
            cwd: dir
        });
        // eslint-disable-next-line no-shadow
        const packagePath = findUp.sync("package.json", {
            cwd: dir
        });
        let bindings = await loadBindings();
        let server = bindings.turbo.startDev({
            ...devServerOptions,
            showAll: args["--show-all"] ?? false,
            root: args["--root"] ?? (turboJson ? _path.default.dirname(turboJson) : packagePath ? _path.default.dirname(packagePath) : undefined)
        });
        // Start preflight after server is listening and ignore errors:
        preflight().catch(()=>{});
        await telemetry.flush();
        return server;
    } else {
        // we're using a sub worker to avoid memory leaks. When memory usage exceeds 90%, we kill the worker and restart it.
        // this is a temporary solution until we can fix the memory leaks.
        // the logic for the worker killing itself is in `packages/next/server/lib/start-server.ts`
        if (!process.env.__NEXT_DISABLE_MEMORY_WATCHER && _cluster.default.isMaster) {
            let config;
            const setupFork = (env)=>{
                const startDir = dir;
                let shouldFilter = false;
                _cluster.default.fork({
                    ...env,
                    FORCE_COLOR: "1"
                });
                // since errors can start being logged from the fork
                // before we detect the project directory rename
                // attempt suppressing them long enough to check
                const filterForkErrors = (chunk, fd)=>{
                    const cleanChunk = (0, _stripAnsi).default(chunk + "");
                    if (cleanChunk.match(/(ENOENT|Module build failed|Module not found|Cannot find module)/)) {
                        if (startDir === dir) {
                            try {
                                var ref;
                                // check if start directory is still valid
                                const result = (0, _findPagesDir).findPagesDir(startDir, !!((ref = config.experimental) == null ? void 0 : ref.appDir));
                                shouldFilter = !Boolean(result.pagesDir || result.appDir);
                            } catch (_) {
                                shouldFilter = true;
                            }
                        }
                        if (shouldFilter || startDir !== dir) {
                            shouldFilter = true;
                            return;
                        }
                    }
                    process[fd].write(chunk);
                };
                for(const workerId in _cluster.default.workers){
                    var ref4, ref1, ref2, ref3;
                    (ref1 = (ref4 = _cluster.default.workers[workerId]) == null ? void 0 : ref4.process.stdout) == null ? void 0 : ref1.on("data", (chunk)=>{
                        filterForkErrors(chunk, "stdout");
                    });
                    (ref3 = (ref2 = _cluster.default.workers[workerId]) == null ? void 0 : ref2.process.stderr) == null ? void 0 : ref3.on("data", (chunk)=>{
                        filterForkErrors(chunk, "stderr");
                    });
                }
            };
            const handleClusterExit = ()=>{
                const callback = async (worker)=>{
                    // ignore if we killed the worker
                    if (worker.killed) return;
                    // TODO: we should track how many restarts are
                    // occurring and how long in-between them
                    if (worker.exitedAfterDisconnect) {
                        setupFork();
                    } else if (!sessionStopHandled) {
                        await handleSessionStop();
                        process.exit(1);
                    }
                };
                _cluster.default.addListener("exit", callback);
                return ()=>_cluster.default.removeListener("exit", callback);
            };
            let clusterExitUnsub = handleClusterExit();
            // x-ref: https://nodejs.org/api/cluster.html#clustersettings
            // @ts-expect-error type is incorrect
            _cluster.default.settings.windowsHide = true;
            _cluster.default.settings.stdio = [
                "ipc",
                "pipe",
                "pipe"
            ];
            setupFork();
            config = await (0, _config).default(_constants.PHASE_DEVELOPMENT_SERVER, dir);
            const handleProjectDirRename = (newDir)=>{
                var ref5;
                clusterExitUnsub();
                for(const workerId in _cluster.default.workers){
                    try {
                        // @ts-expect-error custom field
                        _cluster.default.workers[workerId].killed = true;
                        _cluster.default.workers[workerId].process.kill("SIGKILL");
                    } catch (_) {}
                }
                process.chdir(newDir);
                // @ts-expect-error type is incorrect
                _cluster.default.settings.cwd = newDir;
                _cluster.default.settings.exec = (ref5 = _cluster.default.settings.exec) == null ? void 0 : ref5.replace(dir, newDir);
                setupFork({
                    ...Object.keys(process.env).reduce((newEnv, key)=>{
                        var ref;
                        newEnv[key] = (ref = process.env[key]) == null ? void 0 : ref.replace(dir, newDir);
                        return newEnv;
                    }, {}),
                    NEXT_PRIVATE_DEV_DIR: newDir
                });
                clusterExitUnsub = handleClusterExit();
            };
            const parentDir = _path.default.join("/", dir, "..");
            const watchedEntryLength = parentDir.split("/").length + 1;
            const previousItems = new Set();
            const wp = new _watchpack.default({
                ignored: (entry)=>{
                    // watch only one level
                    return !(entry.split("/").length <= watchedEntryLength);
                }
            });
            wp.watch({
                directories: [
                    parentDir
                ],
                startTime: 0
            });
            wp.on("aggregated", ()=>{
                const knownFiles = wp.getTimeInfoEntries();
                const newFiles = [];
                let hasPagesApp = false;
                // if the dir still exists nothing to check
                try {
                    var ref;
                    const result = (0, _findPagesDir).findPagesDir(dir, !!((ref = config.experimental) == null ? void 0 : ref.appDir));
                    hasPagesApp = Boolean(result.pagesDir || result.appDir);
                } catch (err) {
                    var ref6;
                    // if findPagesDir throws validation error let this be
                    // handled in the dev-server itself in the fork
                    if ((ref6 = err.message) == null ? void 0 : ref6.includes("experimental")) {
                        return;
                    }
                }
                // try to find new dir introduced
                if (previousItems.size) {
                    for (const key of knownFiles.keys()){
                        if (!previousItems.has(key)) {
                            newFiles.push(key);
                        }
                    }
                    previousItems.clear();
                }
                for (const key of knownFiles.keys()){
                    previousItems.add(key);
                }
                if (hasPagesApp) {
                    return;
                }
                // if we failed to find the new dir it may have been moved
                // to a new parent directory which we can't track as easily
                // so exit gracefully
                try {
                    var ref7;
                    const result = (0, _findPagesDir).findPagesDir(newFiles[0], !!((ref7 = config.experimental) == null ? void 0 : ref7.appDir));
                    hasPagesApp = Boolean(result.pagesDir || result.appDir);
                } catch (_) {}
                if (hasPagesApp && newFiles.length === 1) {
                    Log.info(`Detected project directory rename, restarting in new location`);
                    handleProjectDirRename(newFiles[0]);
                    watchConfigFiles(newFiles[0]);
                    dir = newFiles[0];
                } else {
                    Log.error(`Project directory could not be found, restart Next.js in your new directory`);
                    process.exit(0);
                }
            });
        } else {
            (0, _startServer).startServer(devServerOptions).then(async (app)=>{
                const appUrl = `http://${app.hostname}:${app.port}`;
                const hostname = host || "0.0.0.0";
                (0, _output).startedDevelopmentServer(appUrl, `${(0, _net).isIPv6(hostname) ? `[${hostname}]` : hostname}:${app.port}`);
                // Start preflight after server is listening and ignore errors:
                preflight().catch(()=>{});
                // Finalize server bootup:
                await app.prepare();
            }).catch((err)=>{
                if (err.code === "EADDRINUSE") {
                    let errorMessage = `Port ${port} is already in use.`;
                    const pkgAppPath = require("next/dist/compiled/find-up").sync("package.json", {
                        cwd: dir
                    });
                    const appPackage = require(pkgAppPath);
                    if (appPackage.scripts) {
                        const nextScript = Object.entries(appPackage.scripts).find((scriptLine)=>scriptLine[1] === "next");
                        if (nextScript) {
                            errorMessage += `\nUse \`npm run ${nextScript[0]} -- -p <some other port>\`.`;
                        }
                    }
                    console.error(errorMessage);
                } else {
                    console.error(err);
                }
                process.nextTick(()=>process.exit(1));
            });
        }
    }
};
exports.nextDev = nextDev;

//# sourceMappingURL=next-dev.js.map